#!/usr/bin/env python3
# Generates images for the gallery

import os
import argparse
import numpy as np
import os.path as path
import matplotlib.pyplot as plt

from yellowbrick.datasets import load_occupancy, load_credit, load_concrete
from yellowbrick.datasets import load_spam, load_game, load_energy, load_hobbies

from yellowbrick.model_selection import RFECV, FeatureImportances

from yellowbrick.features import PCA, Manifold, JointPlot
from yellowbrick.features import RadViz, Rank1D, Rank2D, ParallelCoordinates

from yellowbrick.contrib.scatter import ScatterVisualizer

from yellowbrick.regressor import ResidualsPlot, PredictionError, AlphaSelection

from yellowbrick.classifier import DiscriminationThreshold
from yellowbrick.classifier import ClassificationReport, ConfusionMatrix
from yellowbrick.classifier import ROCAUC, PRCurve, ClassPredictionError

from yellowbrick.cluster import KElbowVisualizer, SilhouetteVisualizer
from yellowbrick.cluster import InterclusterDistance

from yellowbrick.model_selection import ValidationCurve, LearningCurve, CVScores
from yellowbrick.contrib.classifier import DecisionViz

from yellowbrick.text import (
    FreqDistVisualizer,
    TSNEVisualizer,
    DispersionPlot,
    WordCorrelationPlot,
    PosTagVisualizer,
)

from yellowbrick.target import (
    BalancedBinningReference,
    ClassBalance,
    FeatureCorrelation,
)

from sklearn.tree import DecisionTreeRegressor
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import RidgeClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.cluster import KMeans, MiniBatchKMeans
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.naive_bayes import GaussianNB, MultinomialNB
from sklearn.model_selection import train_test_split as tts
from sklearn.preprocessing import OrdinalEncoder, LabelEncoder
from sklearn.linear_model import Ridge, Lasso, LassoCV, RidgeCV
from sklearn.datasets import load_iris, load_digits, load_diabetes
from sklearn.datasets import make_classification, make_blobs, make_moons
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer


GALLERY = path.join(path.dirname(__file__), "images", "gallery")


##########################################################################
## Helper Methods
##########################################################################


def newfig():
    """
    Helper function to create an axes object of the gallery dimensions.
    """
    # NOTE: this figsize generates a better thumbnail
    _, ax = plt.subplots(figsize=(8, 4))
    return ax


def savefig(viz, name, gallery=GALLERY):
    """
    Saves the figure to the gallery directory
    """
    if not path.exists(gallery):
        os.makedirs(gallery)

    # Must save as png
    if len(name.split(".")) > 1:
        raise ValueError("name should not specify extension")

    outpath = path.join(gallery, name + ".png")
    viz.show(outpath=outpath)
    print("created {}".format(outpath))


##########################################################################
## Feature Analysis
##########################################################################


def radviz():
    X, y = load_occupancy()
    oz = RadViz(ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "radviz")


def rank1d():
    X, y = load_credit()
    oz = Rank1D(algorithm="shapiro", ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "rank1d_shapiro")


def rank2d():
    X, y = load_credit()
    oz = Rank2D(algorithm="covariance", ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "rank2d_covariance")


def pcoords():
    X, y = load_occupancy()
    oz = ParallelCoordinates(sample=0.05, shuffle=True, ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "parallel_coordinates")


def pca():
    X, y = load_credit()
    colors = np.array(["r" if yi else "b" for yi in y])
    oz = PCA(scale=True, color=colors, proj_dim=3)
    oz.fit_transform(X, y)
    savefig(oz, "pca_projection_3d")


def manifold(dataset, manifold):
    if dataset == "concrete":
        X, y = load_concrete()
    elif dataset == "occupancy":
        X, y = load_occupancy()
    else:
        raise ValueError("unknown dataset")

    oz = Manifold(manifold=manifold, ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "{}_{}_manifold".format(dataset, manifold))


def scatter():
    X, y = load_occupancy()
    oz = ScatterVisualizer(x="light", y="CO2", ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "scatter")


def jointplot():
    X, y = load_concrete()
    oz = JointPlot(columns=["cement", "splast"], ax=newfig())
    oz.fit_transform(X, y)
    savefig(oz, "jointplot")


##########################################################################
## Regression
##########################################################################


def residuals():
    X, y = load_concrete()
    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2)
    oz = ResidualsPlot(Ridge(), ax=newfig())
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "residuals")


def peplot():
    X, y = load_concrete()
    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2)
    oz = PredictionError(Lasso(), ax=newfig())
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "prediction_error")


def alphas():
    X, y = load_concrete()
    alphas = np.logspace(-10, 1, 400)
    oz = AlphaSelection(LassoCV(alphas=alphas), ax=newfig())
    oz.fit(X, y)
    savefig(oz, "alpha_selection")


##########################################################################
## Classification
##########################################################################


def classreport():
    X, y = load_occupancy()
    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2)
    oz = ClassificationReport(GaussianNB(), support=True, ax=newfig())
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "classification_report")


def confusion(dataset):
    if dataset == "iris":
        data = load_iris()
    elif dataset == "digits":
        data = load_digits()
    else:
        raise ValueError("uknown dataset")

    X_train, X_test, y_train, y_test = tts(data.data, data.target, test_size=0.2)
    oz = ConfusionMatrix(LogisticRegression(), ax=newfig())
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "confusion_matrix_{}".format(dataset))


def rocauc(dataset):
    if dataset == "binary":
        X, y = load_occupancy()
        model = GaussianNB()
    elif dataset == "multiclass":
        X, y = load_game()
        X = OrdinalEncoder().fit_transform(X)
        model = RidgeClassifier()
    else:
        raise ValueError("uknown dataset")

    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2)
    oz = ROCAUC(model, ax=newfig())
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "rocauc_{}".format(dataset))


def prcurve(dataset):
    if dataset == "binary":
        X, y = load_spam()
        model = RidgeClassifier()
        kws = {}
    elif dataset == "multiclass":
        X, y = load_game()
        X = OrdinalEncoder().fit_transform(X)
        y = LabelEncoder().fit_transform(y)
        model = MultinomialNB()
        kws = {
            "per_class": True,
            "iso_f1_curves": True,
            "fill_area": False,
            "micro": False,
        }
    else:
        raise ValueError("uknown dataset")

    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2, shuffle=True)
    oz = PRCurve(model, ax=newfig(), **kws)
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "precision_recall_{}".format(dataset))


def classprede():
    X, y = make_classification(
        n_samples=1000, n_classes=5, n_informative=3, n_clusters_per_class=1
    )

    classes = ["apple", "kiwi", "pear", "banana", "orange"]

    # Perform 80/20 training/test split
    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.20)
    oz = ClassPredictionError(RandomForestClassifier(), classes=classes, ax=newfig())
    oz.fit(X_train, y_train)
    oz.score(X_test, y_test)
    savefig(oz, "class_prediction_error")


def discrimination():
    X, y = load_spam()
    oz = DiscriminationThreshold(LogisticRegression(solver="lbfgs"), ax=newfig())
    oz.fit(X, y)
    savefig(oz, "discrimination_threshold")


##########################################################################
## Clustering
##########################################################################


def elbow():
    X, _ = make_blobs(centers=8, n_features=12, shuffle=True)
    oz = KElbowVisualizer(KMeans(), k=(4, 12), ax=newfig())
    oz.fit(X)
    savefig(oz, "elbow")


def silhouette():
    X, _ = make_blobs(centers=8)
    oz = SilhouetteVisualizer(MiniBatchKMeans(6), ax=newfig())
    oz.fit(X)
    savefig(oz, "silhouette")


def icdm():
    X, _ = make_blobs(centers=12, n_samples=1000, n_features=16, shuffle=True)
    oz = InterclusterDistance(KMeans(9), ax=newfig())
    oz.fit(X)
    savefig(oz, "icdm")


##########################################################################
## Model Selection
##########################################################################


def validation():
    X, y = load_energy()
    oz = ValidationCurve(
        DecisionTreeRegressor(),
        param_name="max_depth",
        param_range=np.arange(1, 11),
        cv=10,
        scoring="r2",
        ax=newfig(),
    )
    oz.fit(X, y)
    savefig(oz, "validation_curve")


def learning():
    X, y = load_energy()
    sizes = np.linspace(0.3, 1.0, 10)
    oz = LearningCurve(RidgeCV(), train_sizes=sizes, scoring="r2", ax=newfig())
    oz.fit(X, y)
    savefig(oz, "learning_curve")


def cvscores():
    X, y = load_energy()
    oz = CVScores(Ridge(), scoring="r2", cv=10, ax=newfig())
    oz.fit(X, y)
    savefig(oz, "cv_scores")


def decision():
    X, y = make_moons(noise=0.3)
    X = StandardScaler().fit_transform(X)
    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.20)

    oz = DecisionViz(KNeighborsClassifier(3), ax=newfig())
    oz.fit(X_train, y_train)
    oz.draw(X_test, y_test)
    savefig(oz, "decision_boundaries")


def importances():
    X, y = load_occupancy()
    oz = FeatureImportances(RandomForestClassifier(), ax=newfig())
    oz.fit(X, y)
    savefig(oz, "feature_importances")


def rfecv():
    X, y = load_credit()
    model = RandomForestClassifier(n_estimators=10)
    oz = RFECV(model, cv=3, scoring="f1_weighted", ax=newfig())
    oz.fit(X, y)
    savefig(oz, "rfecv_sklearn_example")


##########################################################################
## Text Model Diagnostics
##########################################################################


def freqdist():
    corpus = load_hobbies()
    vecs = CountVectorizer()
    docs = vecs.fit_transform(corpus.data)

    oz = FreqDistVisualizer(features=vecs.get_feature_names(), ax=newfig())
    oz.fit(docs)
    savefig(oz, "freqdist")


def tsne():
    corpus = load_hobbies()
    docs = TfidfVectorizer().fit_transform(corpus.data)

    oz = TSNEVisualizer(ax=newfig())
    oz.fit(docs, corpus.target)
    savefig(oz, "corpus_tsne")


def dispersion():
    corpus = load_hobbies()
    target_words = ["Game", "player", "score", "oil", "Man"]

    oz = DispersionPlot(target_words, ax=newfig())
    oz.fit([doc.split() for doc in corpus.data])
    savefig(oz, "dispersion")


def word_correlation():
    corpus = load_hobbies()
    words = ["Tatsumi Kimishima", "Nintendo", "game", "play", "man", "woman"]

    oz = WordCorrelationPlot(words, ax=newfig())
    oz.fit(corpus.data)
    savefig(oz, "word_correlation")


def postag():
    tagged_stanzas = [
        [
            [
                ("Whose", "JJ"),
                ("woods", "NNS"),
                ("these", "DT"),
                ("are", "VBP"),
                ("I", "PRP"),
                ("think", "VBP"),
                ("I", "PRP"),
                ("know", "VBP"),
                (".", "."),
            ],
            [
                ("His", "PRP$"),
                ("house", "NN"),
                ("is", "VBZ"),
                ("in", "IN"),
                ("the", "DT"),
                ("village", "NN"),
                ("though", "IN"),
                (";", ":"),
                ("He", "PRP"),
                ("will", "MD"),
                ("not", "RB"),
                ("see", "VB"),
                ("me", "PRP"),
                ("stopping", "VBG"),
                ("here", "RB"),
                ("To", "TO"),
                ("watch", "VB"),
                ("his", "PRP$"),
                ("woods", "NNS"),
                ("fill", "VB"),
                ("up", "RP"),
                ("with", "IN"),
                ("snow", "NNS"),
                (".", "."),
            ],
        ],
        [
            [
                ("My", "PRP$"),
                ("little", "JJ"),
                ("horse", "NN"),
                ("must", "MD"),
                ("think", "VB"),
                ("it", "PRP"),
                ("queer", "JJR"),
                ("To", "TO"),
                ("stop", "VB"),
                ("without", "IN"),
                ("a", "DT"),
                ("farmhouse", "NN"),
                ("near", "IN"),
                ("Between", "NNP"),
                ("the", "DT"),
                ("woods", "NNS"),
                ("and", "CC"),
                ("frozen", "JJ"),
                ("lake", "VB"),
                ("The", "DT"),
                ("darkest", "JJS"),
                ("evening", "NN"),
                ("of", "IN"),
                ("the", "DT"),
                ("year", "NN"),
                (".", "."),
            ]
        ],
        [
            [
                ("He", "PRP"),
                ("gives", "VBZ"),
                ("his", "PRP$"),
                ("harness", "NN"),
                ("bells", "VBZ"),
                ("a", "DT"),
                ("shake", "NN"),
                ("To", "TO"),
                ("ask", "VB"),
                ("if", "IN"),
                ("there", "EX"),
                ("is", "VBZ"),
                ("some", "DT"),
                ("mistake", "NN"),
                (".", "."),
            ],
            [
                ("The", "DT"),
                ("only", "JJ"),
                ("other", "JJ"),
                ("sound", "NN"),
                ("’", "NNP"),
                ("s", "VBZ"),
                ("the", "DT"),
                ("sweep", "NN"),
                ("Of", "IN"),
                ("easy", "JJ"),
                ("wind", "NN"),
                ("and", "CC"),
                ("downy", "JJ"),
                ("flake", "NN"),
                (".", "."),
            ],
        ],
        [
            [
                ("The", "DT"),
                ("woods", "NNS"),
                ("are", "VBP"),
                ("lovely", "RB"),
                (",", ","),
                ("dark", "JJ"),
                ("and", "CC"),
                ("deep", "JJ"),
                (",", ","),
                ("But", "CC"),
                ("I", "PRP"),
                ("have", "VBP"),
                ("promises", "NNS"),
                ("to", "TO"),
                ("keep", "VB"),
                (",", ","),
                ("And", "CC"),
                ("miles", "NNS"),
                ("to", "TO"),
                ("go", "VB"),
                ("before", "IN"),
                ("I", "PRP"),
                ("sleep", "VBP"),
                (",", ","),
                ("And", "CC"),
                ("miles", "NNS"),
                ("to", "TO"),
                ("go", "VB"),
                ("before", "IN"),
                ("I", "PRP"),
                ("sleep", "VBP"),
                (".", "."),
            ]
        ],
    ]
    oz = PosTagVisualizer(ax=newfig())
    oz.fit(tagged_stanzas)
    savefig(oz, "postag")


##########################################################################
## Target Visualizations
##########################################################################


def binning():
    _, y = load_concrete()
    oz = BalancedBinningReference(ax=newfig())
    oz.fit(y)
    savefig(oz, "balanced_binning_reference")


def balance():
    X, y = load_occupancy()
    _, _, y_train, y_test = tts(X, y, test_size=0.2)

    oz = ClassBalance(ax=newfig(), labels=["unoccupied", "occupied"])
    oz.fit(y_train, y_test)
    savefig(oz, "class_balance")


def featcorr():
    data = load_diabetes()

    oz = FeatureCorrelation(ax=newfig())
    oz.fit(data.data, data.target)
    savefig(oz, "feature_correlation")


##########################################################################
## Main Method
##########################################################################

if __name__ == "__main__":
    plots = {
        "all": None,
        "radviz": radviz,
        "rank1d": rank1d,
        "rank2d": rank2d,
        "pcoords": pcoords,
        "pca": pca,
        "concrete_tsne": lambda: manifold("concrete", "tsne"),
        "occupancy_tsne": lambda: manifold("occupancy", "tsne"),
        "concrete_isomap": lambda: manifold("concrete", "isomap"),
        "importances": importances,
        "rfecv": rfecv,
        "scatter": scatter,
        "jointplot": jointplot,
        "residuals": residuals,
        "peplot": peplot,
        "alphas": alphas,
        "classreport": classreport,
        "confusion_digits": lambda: confusion("digits"),
        "confusion_iris": lambda: confusion("iris"),
        "rocauc_binary": lambda: rocauc("binary"),
        "rocauc_multi": lambda: rocauc("multiclass"),
        "prcurve_binary": lambda: prcurve("binary"),
        "prcurve_multi": lambda: prcurve("multiclass"),
        "classprede": classprede,
        "discrimination": discrimination,
        "elbow": elbow,
        "silhouette": silhouette,
        "icdm": icdm,
        "validation": validation,
        "learning": learning,
        "cvscores": cvscores,
        "freqdist": freqdist,
        "tsne": tsne,
        "dispersion": dispersion,
        "word_correlation": word_correlation,
        "postag": postag,
        "decision": decision,
        "binning": binning,
        "balance": balance,
        "featcorr": featcorr,
    }

    parser = argparse.ArgumentParser(description="gallery image generator")
    parser.add_argument(
        "plots",
        nargs="+",
        choices=plots.keys(),
        metavar="plot",
        help="names of images to generate",
    )
    args = parser.parse_args()

    queue = frozenset(args.plots)
    if "all" in queue:
        queue = frozenset(plots.keys())

    for item in queue:
        method = plots[item]
        if method is not None:
            method()



# tests.test_regressor.test_influence
# Test the regressor influence visualizers.
#
# Author:   Benjamin Bengfort
# Created:  Sun Jun 09 16:03:31 2019 -0400
#
# Copyright (C) 2019 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_influence.py [fe14cfd] benjamin@bengfort.com $

"""
Test the regressor influence visualizers.
"""

##########################################################################
## Imports
##########################################################################

import pytest
import numpy as np
import matplotlib.pyplot as plt

from tests.base import VisualTestCase
from tests.fixtures import Dataset
from sklearn.datasets import make_regression

from yellowbrick.regressor.influence import *
from yellowbrick.datasets import load_concrete

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
## Fixtures
##########################################################################


@pytest.fixture(scope="class")
def data(request):
    """
    Creates a random regression fixture that has a R2 score below 0.85 and several
    outliers that best demonstrate the effectiveness of influence visualizers.
    """
    X, y = make_regression(
        n_samples=100,
        n_features=14,
        n_informative=6,
        bias=1.2,
        noise=49.8,
        tail_strength=0.6,
        random_state=637,
    )

    request.cls.data = Dataset(X, y)


##########################################################################
## Assertion Helpers
##########################################################################

LEARNED_FIELDS = (
    "distance_",
    "p_values_",
    "influence_threshold_",
    "outlier_percentage_",
)


def assert_not_fitted(oz):
    for field in LEARNED_FIELDS:
        assert not hasattr(oz, field)


def assert_fitted(oz):
    for field in LEARNED_FIELDS:
        assert hasattr(oz, field)


##########################################################################
## Test CooksDistance Visualizer
##########################################################################


@pytest.mark.usefixtures("data")
class TestCooksDistance(VisualTestCase):
    """
    CooksDistance visual test cases
    """

    def test_cooks_distance(self):
        """
        Test image similarity of Cook's Distance on a random dataset
        """
        _, ax = plt.subplots()
        viz = CooksDistance(ax=ax)

        assert_not_fitted(viz)
        assert viz.fit(self.data.X, self.data.y) is viz
        assert_fitted(viz)

        # Test fitted values
        assert viz.distance_.shape == (self.data.X.shape[0],)
        assert viz.p_values_.shape == viz.distance_.shape
        assert 0.0 <= viz.influence_threshold_ <= 4.0
        assert 0.0 <= viz.outlier_percentage_ <= 100.0

        self.assert_images_similar(viz)

    def test_cooks_distance_quickmethod(self):
        """
        Test the cooks_distance quick method on a random dataset
        """
        _, ax = plt.subplots()
        viz = cooks_distance(
            self.data.X,
            self.data.y,
            ax=ax,
            draw_threshold=False,
            linefmt="r-",
            markerfmt="ro",
            show=False
        )

        assert_fitted(viz)
        self.assert_images_similar(viz)

    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_integration(self):
        """
        Test on the concrete dataset with pandas DataFrame and Series
        """
        data = load_concrete(return_dataset=True)
        X, y = data.to_pandas()

        assert isinstance(X, pd.DataFrame)
        assert isinstance(y, pd.Series)

        _, ax = plt.subplots()
        viz = CooksDistance(ax=ax).fit(X, y)
        assert_fitted(viz)

        assert viz.distance_.sum() == pytest.approx(1.2911900571300652)
        assert viz.p_values_.sum() == pytest.approx(1029.9999525376425)
        assert viz.influence_threshold_ == pytest.approx(0.003883495145631068)
        assert viz.outlier_percentage_ == pytest.approx(7.3786407766990285)

        viz.finalize()
        self.assert_images_similar(viz)

    def test_numpy_integration(self):
        """
        Test on concrete dataset with numpy arrays
        """
        data = load_concrete(return_dataset=True)
        X, y = data.to_numpy()

        assert isinstance(X, np.ndarray)
        assert isinstance(y, np.ndarray)

        _, ax = plt.subplots()
        viz = CooksDistance(ax=ax).fit(X, y)
        assert_fitted(viz)

        assert viz.distance_.sum() == pytest.approx(1.2911900571300652)
        assert viz.p_values_.sum() == pytest.approx(1029.9999525376425)
        assert viz.influence_threshold_ == pytest.approx(0.003883495145631068)
        assert viz.outlier_percentage_ == pytest.approx(7.3786407766990285)

        viz.finalize()
        self.assert_images_similar(viz)



#!/usr/bin/env python3
# ID: umap_vis.py [73a44e5] jchealy@gmail.com $

"""
Manually generate figures for the UMAP documentation.
"""
##########################################################################
## Imports
##########################################################################

import matplotlib.pyplot as plt

from sklearn.cluster import KMeans
from sklearn.feature_extraction.text import TfidfVectorizer

from yellowbrick.text import UMAPVisualizer
from yellowbrick.datasets import load_hobbies

##########################################################################
## Generate
##########################################################################


def umap(docs, target, outpath, **kwargs):
    # Create a new figure and axes
    fig = plt.figure()
    ax = fig.add_subplot(111)

    # Visualize the frequency distribution
    visualizer = UMAPVisualizer(ax=ax, **kwargs)
    visualizer.fit(docs, target)
    visualizer.show(outpath=outpath)


if __name__ == "__main__":

    # Load and vectorize the corpus
    corpus = load_hobbies()
    tfidf = TfidfVectorizer()

    docs = tfidf.fit_transform(corpus.data)
    labels = corpus.target

    # Whole corpus visualization
    umap(docs, labels, "images/umap.png")

    # Whole corpus visualization
    umap(docs, labels, "images/umap_cosine.png", metric="cosine")

    # No labels
    umap(docs, None, "images/umap_no_labels.png", labels=["documents"], metric="cosine")

    # Apply clustering instead of class names
    clusters = KMeans(n_clusters=5)
    clusters.fit(docs)

    centers = ["c{}".format(c) for c in clusters.labels_]
    umap(docs, centers, "images/umap_kmeans.png")



#!/usr/bin/env python3
# Script to create visualizations for the JOSS paper

import os
import argparse
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from yellowbrick.features import Rank2D, RadViz
from yellowbrick.model_selection import LearningCurve
from yellowbrick.cluster import KElbowVisualizer, SilhouetteVisualizer
from yellowbrick.classifier import ClassificationReport, DiscriminationThreshold
from yellowbrick.regressor import ResidualsPlot, PredictionError, AlphaSelection

from collections import namedtuple
from sklearn.datasets import make_blobs
from sklearn.naive_bayes import MultinomialNB
from sklearn.ensemble import RandomForestRegressor
from sklearn.cluster import MiniBatchKMeans, Birch
from sklearn.model_selection import train_test_split as tts
from sklearn.linear_model import LassoCV, RidgeCV, LogisticRegression


# Store figures alongside the script that generates them
FIGURES = os.path.dirname(__file__)

# Path to datasets downloaded from S3
DATA = os.path.join(
    os.path.dirname(__file__), "..", "..", "yellowbrick", "datasets", "fixtures"
)

# Quick reference dataset objects
Dataset = namedtuple("Dataset", "X,y")
Split = namedtuple("Split", "train,test")


def _make_dataset(X, y, split=False):
    if split:
        X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2)
        return Dataset(Split(X_train, X_test), Split(y_train, y_test))
    return Dataset(X, y)


def load_occupancy(split=False):
    """
    Create a dataset for the specified yb dataset
    """
    path = os.path.join(DATA, "occupancy", "occupancy.csv")
    data = pd.read_csv(path)

    X = data[["temperature", "relative humidity", "light", "C02", "humidity"]]
    y = data["occupancy"]
    return _make_dataset(X, y, split)


def load_concrete(split=False):
    path = os.path.join(DATA, "concrete", "concrete.csv")
    data = pd.read_csv(path)

    X = data[["cement", "slag", "ash", "water", "splast", "coarse", "fine", "age"]]
    y = data["strength"]
    return _make_dataset(X, y, split)


def load_spam(split=False):
    path = os.path.join(DATA, "spam", "spam.csv")
    data = pd.read_csv(path)

    target = "is_spam"
    features = [col for col in data.columns if col != target]

    X = data[features]
    y = data[target]
    return _make_dataset(X, y, split)


def feature_analysis(fname="feature_analysis.png"):
    """
    Create figures for feature analysis
    """

    # Create side-by-side axes grid
    _, axes = plt.subplots(ncols=2, figsize=(18, 6))

    # Draw RadViz on the left
    data = load_occupancy(split=False)
    oz = RadViz(ax=axes[0], classes=["unoccupied", "occupied"])
    oz.fit(data.X, data.y)
    oz.finalize()

    # Draw Rank2D on the right
    data = load_concrete(split=False)
    oz = Rank2D(ax=axes[1])
    oz.fit_transform(data.X, data.y)
    oz.finalize()

    # Save figure
    path = os.path.join(FIGURES, fname)
    plt.tight_layout()
    plt.savefig(path)


def regression(fname="regression.png"):
    """
    Create figures for regression models
    """
    _, axes = plt.subplots(ncols=2, figsize=(18, 6))
    alphas = np.logspace(-10, 1, 300)
    data = load_concrete(split=True)

    # Plot prediction error in the middle
    oz = PredictionError(LassoCV(alphas=alphas), ax=axes[0])
    oz.fit(data.X.train, data.y.train)
    oz.score(data.X.test, data.y.test)
    oz.finalize()

    # Plot residuals on the right
    oz = ResidualsPlot(RidgeCV(alphas=alphas), ax=axes[1])
    oz.fit(data.X.train, data.y.train)
    oz.score(data.X.test, data.y.test)
    oz.finalize()

    # Save figure
    path = os.path.join(FIGURES, fname)
    plt.tight_layout()
    plt.savefig(path)


def classification(fname="classification.png"):

    # Create side-by-side axes grid
    _, axes = plt.subplots(ncols=2, figsize=(18, 6))

    # Add ClassificationReport to the reft
    data = load_spam(split=True)
    oz = ClassificationReport(MultinomialNB(), classes=["ham", "spam"], ax=axes[0])
    oz.fit(data.X.train, data.y.train)
    oz.score(data.X.test, data.y.test)
    oz.finalize()

    # Add DiscriminationThreshold to the right
    data = load_spam(split=False)
    oz = DiscriminationThreshold(LogisticRegression(), ax=axes[1])
    oz.fit(data.X, data.y)
    oz.finalize()

    # Save figure
    path = os.path.join(FIGURES, fname)
    plt.tight_layout()
    plt.savefig(path)


def clustering(fname="clustering.png"):
    # Create side-by-side axes grid
    _, axes = plt.subplots(ncols=2, figsize=(18, 6))
    X, y = make_blobs(centers=7)

    # Add K-Elbow to the left
    oz = KElbowVisualizer(MiniBatchKMeans(), k=(3, 12), ax=axes[0])
    oz.fit(X, y)
    oz.finalize()

    # Add SilhouetteVisualizer to the right
    oz = SilhouetteVisualizer(Birch(n_clusters=5), ax=axes[1])
    oz.fit(X, y)
    oz.finalize()

    # Save figure
    path = os.path.join(FIGURES, fname)
    plt.tight_layout()
    plt.savefig(path)


def hyperparameter_tuning(fname="hyperparameter_tuning.png"):
    # Create side-by-side axes grid
    _, axes = plt.subplots(ncols=2, figsize=(18, 6))

    # Load the concrete dataset
    data = load_concrete(split=False)

    # Create a list of alphas to cross-validate against
    alphas = np.logspace(-10, 1, 400)

    # Add AlphaSelection to the left
    oz = AlphaSelection(LassoCV(alphas=alphas), ax=axes[0])
    oz.fit(data.X, data.y)
    oz.finalize()

    # Add LearningCurve to the right
    oz = LearningCurve(RandomForestRegressor(), scoring="r2", ax=axes[1])
    oz.fit(data.X, data.y)
    oz.finalize()

    # Save figure
    path = os.path.join(FIGURES, fname)
    plt.tight_layout()
    plt.savefig(path)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="generate visualizations for JOSS paper"
    )

    args = parser.parse_args()
    feature_analysis()
    regression()
    classification()
    clustering()
    hyperparameter_tuning()



# tests.test_utils.test_types
# Tests for type checking utilities and validation
#
# Author:   Benjamin Bengfort <bbengfort@districtdatalabs.com>
# Created:  Fri May 19 10:58:32 2017 -0700
#
# ID: test_types.py [79cd8cf] benjamin@bengfort.com $

"""
Tests for type checking utilities and validation.

Generally if there is a problem with a type checking utility, the offending
object should be imported then added to the correct bucket under the import
statement (e.g. REGRESSORS). The pytest parametrize decorator uses these
groups to generate tests, so this will automatically cause the test to run on
that class.
"""

##########################################################################
## Imports
##########################################################################

import pytest
import inspect

try:
    import pandas as pd
except:
    pd = None

# Yellowbrick Utilities
from yellowbrick.utils.types import *
from yellowbrick.base import Visualizer, ScoreVisualizer, ModelVisualizer

# Import Regressors
from sklearn.svm import SVR
from sklearn.tree import DecisionTreeRegressor
from sklearn.neural_network import MLPRegressor
from sklearn.linear_model import LinearRegression
from sklearn.ensemble import RandomForestRegressor
from sklearn.linear_model import Ridge, RidgeCV, Lasso, LassoCV

REGRESSORS = [
    SVR,
    DecisionTreeRegressor,
    MLPRegressor,
    LinearRegression,
    RandomForestRegressor,
    Ridge,
    RidgeCV,
    Lasso,
    LassoCV,
]

# Import Classifiers
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.naive_bayes import MultinomialNB, GaussianNB

CLASSIFIERS = [
    SVC,
    DecisionTreeClassifier,
    MLPClassifier,
    LogisticRegression,
    RandomForestClassifier,
    GradientBoostingClassifier,
    MultinomialNB,
    GaussianNB,
]

# Import Clusterers
from sklearn.cluster import KMeans, MiniBatchKMeans
from sklearn.cluster import AffinityPropagation, Birch

CLUSTERERS = [KMeans, MiniBatchKMeans, AffinityPropagation, Birch]

# Import Decompositions
from sklearn.decomposition import PCA
from sklearn.decomposition import TruncatedSVD

DECOMPOSITIONS = [PCA, TruncatedSVD]

# Import Transformers
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.feature_extraction import DictVectorizer
from sklearn.preprocessing import QuantileTransformer
from sklearn.preprocessing import StandardScaler
from sklearn.impute import SimpleImputer

TRANSFORMERS = [
    DictVectorizer,
    QuantileTransformer,
    StandardScaler,
    SimpleImputer,
    TfidfVectorizer,
]

# Import Pipeline Utilities
from sklearn.pipeline import Pipeline, FeatureUnion


PIPELINES = [Pipeline, FeatureUnion]

# Import GridSearch Utilities
from sklearn.model_selection import GridSearchCV, RandomizedSearchCV

SEARCH = [GridSearchCV, RandomizedSearchCV]


# Other Groups
MODELS = REGRESSORS + CLASSIFIERS + CLUSTERERS
ESTIMATORS = MODELS + DECOMPOSITIONS + TRANSFORMERS


# Get the name of the object to label test cases
def obj_name(obj):
    if inspect.isclass(obj):
        return obj.__name__
    return obj.__class__.__name__


##########################################################################
## Model type checking test cases
##########################################################################


class TestModelTypeChecking(object):
    """
    Test model type checking utilities
    """

    ##////////////////////////////////////////////////////////////////////
    ## is_estimator testing
    ##////////////////////////////////////////////////////////////////////

    def test_estimator_alias(self):
        """
        Assert isestimator aliases is_estimator
        """
        assert isestimator is is_estimator

    @pytest.mark.parametrize("model", ESTIMATORS, ids=obj_name)
    def test_is_estimator(self, model):
        """
        Test that is_estimator works for instances and classes
        """
        assert inspect.isclass(model)
        assert is_estimator(model)

        obj = model()
        assert is_estimator(obj)

    @pytest.mark.parametrize(
        "cls", [list, dict, tuple, set, str, bool, int, float], ids=obj_name
    )
    def test_not_is_estimator(self, cls):
        """
        Assert Python objects are not estimators
        """
        assert inspect.isclass(cls)
        assert not is_estimator(cls)

        obj = cls()
        assert not is_estimator(obj)

    def test_is_estimator_pipeline(self):
        """
        Test that is_estimator works for pipelines
        """
        assert is_estimator(Pipeline)
        assert is_estimator(FeatureUnion)

        model = Pipeline([("reduce_dim", PCA()), ("linreg", LinearRegression())])

        assert is_estimator(model)

    def test_is_estimator_search(self):
        """
        Test that is_estimator works for search
        """
        assert is_estimator(GridSearchCV)
        assert is_estimator(RandomizedSearchCV)

        model = GridSearchCV(SVR(), {"kernel": ["linear", "rbf"]})
        assert is_estimator(model)

    @pytest.mark.parametrize(
        "viz,params",
        [
            (Visualizer, {}),
            (ScoreVisualizer, {"estimator": LinearRegression()}),
            (ModelVisualizer, {"estimator": LogisticRegression()}),
        ],
        ids=["Visualizer", "ScoreVisualizer", "ModelVisualizer"],
    )
    def test_is_estimator_visualizer(self, viz, params):
        """
        Test that is_estimator works for Visualizers
        """
        assert inspect.isclass(viz)
        assert is_estimator(viz)

        obj = viz(**params)
        assert is_estimator(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_regressor testing
    ##////////////////////////////////////////////////////////////////////

    def test_regressor_alias(self):
        """
        Assert isregressor aliases is_regressor
        """
        assert isregressor is is_regressor

    @pytest.mark.parametrize("model", REGRESSORS, ids=obj_name)
    def test_is_regressor(self, model):
        """
        Test that is_regressor works for instances and classes
        """
        assert inspect.isclass(model)
        assert is_regressor(model)

        obj = model()
        assert is_regressor(obj)

    @pytest.mark.parametrize(
        "model", CLASSIFIERS + CLUSTERERS + TRANSFORMERS + DECOMPOSITIONS, ids=obj_name
    )
    def test_not_is_regressor(self, model):
        """
        Test that is_regressor does not match non-regressor estimators
        """
        assert inspect.isclass(model)
        assert not is_regressor(model)

        obj = model()
        assert not is_regressor(obj)

    def test_is_regressor_pipeline(self):
        """
        Test that is_regressor works for pipelines
        """
        assert not is_regressor(Pipeline)
        assert not is_regressor(FeatureUnion)

        model = Pipeline([("reduce_dim", PCA()), ("linreg", LinearRegression())])

        assert is_regressor(model)

    @pytest.mark.xfail(reason="grid search has no _estimator_type it seems")
    def test_is_regressor_search(self):
        """
        Test that is_regressor works for search
        """
        assert is_regressor(GridSearchCV)
        assert is_regressor(RandomizedSearchCV)

        model = GridSearchCV(SVR(), {"kernel": ["linear", "rbf"]})
        assert is_regressor(model)

    @pytest.mark.parametrize(
        "viz,params",
        [
            (ScoreVisualizer, {"estimator": LinearRegression()}),
            (ModelVisualizer, {"estimator": Ridge()}),
        ],
        ids=["ScoreVisualizer", "ModelVisualizer"],
    )
    def test_is_regressor_visualizer(self, viz, params):
        """
        Test that is_regressor works on visualizers
        """
        assert inspect.isclass(viz)
        assert not is_regressor(viz)

        obj = viz(**params)
        assert is_regressor(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_classifier testing
    ##////////////////////////////////////////////////////////////////////

    def test_classifier_alias(self):
        """
        Assert isclassifier aliases is_classifier
        """
        assert isclassifier is is_classifier

    @pytest.mark.parametrize("model", CLASSIFIERS, ids=obj_name)
    def test_is_classifier(self, model):
        """
        Test that is_classifier works for instances and classes
        """
        assert inspect.isclass(model)
        assert is_classifier(model)

        obj = model()
        assert is_classifier(obj)

    @pytest.mark.parametrize(
        "model", REGRESSORS + CLUSTERERS + TRANSFORMERS + DECOMPOSITIONS, ids=obj_name
    )
    def test_not_is_classifier(self, model):
        """
        Test that is_classifier does not match non-classifier estimators
        """
        assert inspect.isclass(model)
        assert not is_classifier(model)

        obj = model()
        assert not is_classifier(obj)

    def test_classifier_pipeline(self):
        """
        Test that is_classifier works for pipelines
        """
        assert not is_classifier(Pipeline)
        assert not is_classifier(FeatureUnion)

        model = Pipeline([("reduce_dim", PCA()), ("linreg", LogisticRegression())])

        assert is_classifier(model)

    @pytest.mark.xfail(reason="grid search has no _estimator_type it seems")
    def test_is_classifier_search(self):
        """
        Test that is_classifier works for search
        """
        assert is_classifier(GridSearchCV)
        assert is_classifier(RandomizedSearchCV)

        model = GridSearchCV(SVC(), {"kernel": ["linear", "rbf"]})
        assert is_classifier(model)

    @pytest.mark.parametrize(
        "viz,params",
        [
            (ScoreVisualizer, {"estimator": MultinomialNB()}),
            (ModelVisualizer, {"estimator": MLPClassifier()}),
        ],
        ids=["ScoreVisualizer", "ModelVisualizer"],
    )
    def test_is_classifier_visualizer(self, viz, params):
        """
        Test that is_classifier works on visualizers
        """
        assert inspect.isclass(viz)
        assert not is_classifier(viz)

        obj = viz(**params)
        assert is_classifier(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_clusterer testing
    ##////////////////////////////////////////////////////////////////////

    def test_clusterer_alias(self):
        """
        Assert isclusterer aliases is_clusterer
        """
        assert isclusterer is is_clusterer

    @pytest.mark.parametrize("model", CLUSTERERS, ids=obj_name)
    def test_is_clusterer(self, model):
        """
        Test that is_clusterer works for instances and classes
        """
        assert inspect.isclass(model)
        assert is_clusterer(model)

        obj = model()
        assert is_clusterer(obj)

    @pytest.mark.parametrize(
        "model", REGRESSORS + CLASSIFIERS + TRANSFORMERS + DECOMPOSITIONS, ids=obj_name
    )
    def test_not_is_clusterer(self, model):
        """
        Test that is_clusterer does not match non-clusterer estimators
        """
        assert inspect.isclass(model)
        assert not is_clusterer(model)

        obj = model()
        assert not is_clusterer(obj)

    def test_clusterer_pipeline(self):
        """
        Test that is_clusterer works for pipelines
        """
        assert not is_clusterer(Pipeline)
        assert not is_clusterer(FeatureUnion)

        model = Pipeline([("reduce_dim", PCA()), ("kmeans", KMeans())])

        assert is_clusterer(model)

    @pytest.mark.parametrize(
        "viz,params", [
            (ModelVisualizer, {"estimator": KMeans()})
        ], ids=["ModelVisualizer"]
    )
    def test_is_clusterer_visualizer(self, viz, params):
        """
        Test that is_clusterer works on visualizers
        """
        assert inspect.isclass(viz)
        assert not is_clusterer(viz)

        obj = viz(**params)
        assert is_clusterer(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_gridsearch testing
    ##////////////////////////////////////////////////////////////////////

    def test_gridsearch_alias(self):
        """
        Assert isgridsearch aliases is_gridsearch
        """
        assert isgridsearch is is_gridsearch

    @pytest.mark.parametrize("model", SEARCH, ids=obj_name)
    def test_is_gridsearch(self, model):
        """
        Test that is_gridsearch works correctly
        """
        assert inspect.isclass(model)
        assert is_gridsearch(model)

        obj = model(SVC, {"C": [0.5, 1, 10]})
        assert is_gridsearch(obj)

    @pytest.mark.parametrize(
        "model", [MLPRegressor, MLPClassifier, SimpleImputer], ids=obj_name
    )
    def test_not_is_gridsearch(self, model):
        """
        Test that is_gridsearch does not match non grid searches
        """
        assert inspect.isclass(model)
        assert not is_gridsearch(model)

        obj = model()
        assert not is_gridsearch(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_probabilistic testing
    ##////////////////////////////////////////////////////////////////////

    def test_probabilistic_alias(self):
        """
        Assert isprobabilistic aliases is_probabilistic
        """
        assert isprobabilistic is is_probabilistic

    @pytest.mark.parametrize(
        "model",
        [
            MultinomialNB,
            GaussianNB,
            LogisticRegression,
            SVC,
            RandomForestClassifier,
            GradientBoostingClassifier,
            MLPClassifier,
        ],
        ids=obj_name,
    )
    def test_is_probabilistic(self, model):
        """
        Test that is_probabilistic works correctly
        """
        assert inspect.isclass(model)
        assert is_probabilistic(model)

        obj = model()
        assert is_probabilistic(obj)

    @pytest.mark.parametrize(
        "model",
        [MLPRegressor, SimpleImputer, StandardScaler, KMeans, RandomForestRegressor],
        ids=obj_name,
    )
    def test_not_is_probabilistic(self, model):
        """
        Test that is_probabilistic does not match non probablistic estimators
        """
        assert inspect.isclass(model)
        assert not is_probabilistic(model)

        obj = model()
        assert not is_probabilistic(obj)


##########################################################################
## Data type checking test cases
##########################################################################


class TestDataTypeChecking(object):
    """
    Test data type checking utilities
    """

    ##////////////////////////////////////////////////////////////////////
    ## is_dataframe testing
    ##////////////////////////////////////////////////////////////////////

    def test_dataframe_alias(self):
        """
        Assert isdataframe aliases is_dataframe
        """
        assert isdataframe is is_dataframe

    @pytest.mark.skipif(pd is None, reason="requires pandas")
    def test_is_dataframe(self):
        """
        Test that is_dataframe works correctly
        """
        df = pd.DataFrame(
            [{"a": 1, "b": 2.3, "c": "Hello"}, {"a": 2, "b": 3.14, "c": "World"}]
        )

        assert is_dataframe(df)

    @pytest.mark.parametrize(
        "obj",
        [
            np.array(
                [(1, 2.0, "Hello"), (2, 3.0, "World")],
                dtype=[("foo", "i4"), ("bar", "f4"), ("baz", "S10")],
            ),
            np.array([[1, 2, 3], [1, 2, 3]]),
            [[1, 2, 3], [1, 2, 3]],
        ],
        ids=["structured array", "array", "list"],
    )
    def test_not_is_dataframe(self, obj):
        """
        Test that is_dataframe does not match non-dataframes
        """
        assert not is_dataframe(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_series testing
    ##////////////////////////////////////////////////////////////////////

    def test_series_alias(self):
        """
        Assert isseries aliases is_series
        """
        assert isseries is is_series

    @pytest.mark.skipif(pd is None, reason="requires pandas")
    def test_is_series(self):
        """
        Test that is_series works correctly
        """
        df = pd.Series([1, 2, 3])

        assert is_series(df)

    @pytest.mark.parametrize(
        "obj",
        [
            np.array(
                [(1, 2.0, "Hello"), (2, 3.0, "World")],
                dtype=[("foo", "i4"), ("bar", "f4"), ("baz", "S10")],
            ),
            np.array([1, 2, 3]),
            [1, 2, 3],
        ],
        ids=["structured array", "array", "list"],
    )
    def test_not_is_series(self, obj):
        """
        Test that is_series does not match non-dataframes
        """
        assert not is_series(obj)

    ##////////////////////////////////////////////////////////////////////
    ## is_structured_array testing
    ##////////////////////////////////////////////////////////////////////

    def test_structured_array_alias(self):
        """
        Assert isstructuredarray aliases is_structured_array
        """
        assert isstructuredarray is is_structured_array

    def test_is_structured_array(self):
        """
        Test that is_structured_array works correctly
        """
        x = np.array(
            [(1, 2.0, "Hello"), (2, 3.0, "World")],
            dtype=[("foo", "i4"), ("bar", "f4"), ("baz", "S10")],
        )

        assert is_structured_array(x)

    @pytest.mark.parametrize(
        "obj", [np.array([[1, 2, 3], [1, 2, 3]]), [[1, 2, 3], [1, 2, 3]]], ids=obj_name
    )
    def test_not_is_structured_array(self, obj):
        """
        Test that is_structured_array does not match non-structured-arrays
        """
        assert not is_structured_array(obj)



# tests.test_draw
# Tests for the high-level drawing utility functions
#
# Author:  Benjamin Bengfort <benjamin@bengfort.com>
# Created: Sun Aug 19 11:21:04 2018 -0400
#
# ID: test_draw.py [dd915ad] benjamin@bengfort.com $

"""
Tests for the high-level drawing utility functions
"""

##########################################################################
## Imports
##########################################################################

import pytest
import numpy as np
import matplotlib.pyplot as plt

from yellowbrick.draw import *
from .base import VisualTestCase


##########################################################################
## Simple tests for high-level drawing utilities
##########################################################################


def test_manual_legend_uneven_colors():
    """
    Raise exception when colors and labels are mismatched in manual_legend
    """
    with pytest.raises(YellowbrickValueError, match="same number of colors as labels"):
        manual_legend(None, ("a", "b", "c"), ("r", "g"))


@pytest.fixture(scope="class")
def data(request):

    data = np.array(
        [
            [4, 8, 7, 6, 5, 2, 1],
            [6, 7, 9, 6, 9, 3, 6],
            [5, 1, 6, 8, 4, 7, 8],
            [6, 8, 1, 5, 6, 7, 4],
        ]
    )

    request.cls.data = data


##########################################################################
## Visual test cases for high-level drawing utilities
##########################################################################


@pytest.mark.usefixtures("data")
class TestDraw(VisualTestCase):
    """
    Visual tests for the high-level drawing utilities
    """

    def test_manual_legend(self):
        """
        Check that the manual legend is drawn without axes artists
        """
        # Draw a random scatter plot
        random = np.random.RandomState(42)

        Ax, Ay = random.normal(50, 2, 100), random.normal(50, 3, 100)
        Bx, By = random.normal(42, 3, 100), random.normal(44, 1, 100)
        Cx, Cy = random.normal(20, 10, 100), random.normal(30, 1, 100)

        _, ax = plt.subplots()
        ax.scatter(Ax, Ay, c="r", alpha=0.35, label="a")
        ax.scatter(Bx, By, c="g", alpha=0.35, label="b")
        ax.scatter(Cx, Cy, c="b", alpha=0.35, label="c")

        # Add the manual legend
        manual_legend(
            ax, ("a", "b", "c"), ("r", "g", "b"), frameon=True, loc="upper left"
        )

        # Assert image similarity
        self.assert_images_similar(ax=ax, tol=0.5)

    def test_vertical_bar_stack(self):
        """
        Test bar_stack for vertical orientation
        """
        _, ax = plt.subplots()

        # Plots stacked bar charts
        bar_stack(self.data, ax=ax, orientation="v")

        # Assert image similarity
        self.assert_images_similar(ax=ax, tol=0.1)

    def test_horizontal_bar_stack(self):
        """
        Test bar_stack for horizontal orientation
        """
        _, ax = plt.subplots()
        # Plots stacked bar charts
        bar_stack(self.data, ax=ax, orientation="h")

        # Assert image similarity
        self.assert_images_similar(ax=ax, tol=0.1)

    def test_single_row_bar_stack(self):
        """
        Test bar_stack for single row
        """
        data = np.array([[4, 8, 7, 6, 5, 2, 1]])

        _, ax = plt.subplots()

        # Plots stacked bar charts
        bar_stack(data, ax=ax)

        # Assert image similarity
        self.assert_images_similar(ax=ax, tol=0.1)

    def test_labels_vertical(self):
        """
        Test labels and ticks for vertical barcharts
        """
        labels = ["books", "cinema", "cooking", "gaming"]
        ticks = ["noun", "verb", "adverb", "pronoun", "preposition", "digit", "other"]
        _, ax = plt.subplots()

        # Plots stacked bar charts
        bar_stack(self.data, labels=labels, ticks=ticks, colors=["r", "b", "g", "y"])

        # Extract tick labels from the plot
        ticks_ax = [tick.get_text() for tick in ax.xaxis.get_ticklabels()]
        # Assert that ticks are set properly
        assert ticks_ax == ticks

        # Assert image similarity
        self.assert_images_similar(ax=ax, tol=0.05)

    def test_labels_horizontal(self):
        """
        Test labels and ticks with horizontal barcharts
        """
        labels = ["books", "cinema", "cooking", "gaming"]
        ticks = ["noun", "verb", "adverb", "pronoun", "preposition", "digit", "other"]
        _, ax = plt.subplots()

        # Plots stacked bar charts
        bar_stack(
            self.data, labels=labels, ticks=ticks, orientation="h", colormap="cool"
        )

        # Extract tick labels from the plot
        ticks_ax = [tick.get_text() for tick in ax.yaxis.get_ticklabels()]
        # Assert that ticks are set properly
        assert ticks_ax == ticks

        # Assert image similarity
        self.assert_images_similar(ax=ax, tol=0.05)
